@@ -18,7 +18,13 @@ from coiled.software_utils import (
     partition_local_packages,
     partition_local_python_code_packages,
 )
-from coiled.types import ArchitectureTypesEnum, PackageInfo, PackageLevelEnum, ResolvedPackageInfo, parse_conda_channel
+from coiled.types import (
+    ArchitectureTypesEnum,
+    PackageInfo,
+    PackageLevelEnum,
+    ResolvedPackageInfo,
+    parse_conda_channel,
+)
 from coiled.v2.core import CloudV2
 from coiled.v2.widgets.util import simple_progress
 
@@ -112,11 +118,14 @@ async def approximate_packages(
                     "priority_override": (
                         PackageLevelEnum.CRITICAL
                         if (
-                            strict
-                            or (
-                                pkg["wheel_target"]
-                                # Ignore should override wheel_target (see #2640)
-                                and not priorities.get((pkg["name"], pkg["source"])) == PackageLevelEnum.IGNORE
+                            pkg["version"]
+                            and (
+                                strict
+                                or (
+                                    pkg["wheel_target"]
+                                    # Ignore should override wheel_target (see #2640)
+                                    and not priorities.get((pkg["name"], pkg["source"])) == PackageLevelEnum.IGNORE
+                                )
                             )
                         )
                         else priorities.get((
@@ -135,7 +144,8 @@ async def approximate_packages(
                     "version": pkg["version"],
                     "wheel_target": pkg["wheel_target"],
                 }
-                for pkg in packages
+                # Send all packages to backend to help with debugging
+                for pkg in packages + local_python_code + local_python_wheel_packages
             ],
             architecture=architecture,
             pip_check_errors=pip_check_errors,
@@ -181,6 +191,7 @@ async def create_environment_approximation(
     cloud: CloudV2,
     priorities: Dict[Tuple[str, Literal["conda", "pip"]], PackageLevelEnum],
     only: Optional[Set[str]] = None,
+    conda_extras: Optional[List[str]] = None,
     strict: bool = False,
     progress: Optional[Progress] = None,
     architecture: ArchitectureTypesEnum = ArchitectureTypesEnum.X86_64,
@@ -190,9 +201,23 @@ async def create_environment_approximation(
     pip_check_errors = await check_pip_happy(progress)
     if only:
         packages = [pkg for pkg in packages if pkg["name"] in only]
+    extra_packages: List[PackageInfo] = [
+        {
+            "name": conda_extra,
+            "path": None,
+            "source": "conda",
+            "channel_url": ANY_AVAILABLE,
+            "channel": ANY_AVAILABLE,
+            "subdir": f"linux-{architecture.conda_suffix}",
+            "conda_name": conda_extra,
+            "version": "",
+            "wheel_target": None,
+        }
+        for conda_extra in (conda_extras or [])
+    ]
     result = await approximate_packages(
         cloud=cloud,
-        packages=[pkg for pkg in packages],
+        packages=[pkg for pkg in packages] + extra_packages,
         priorities=priorities,
         strict=strict,
         progress=progress,
