@@ -43,22 +43,28 @@ def pull_azure_logs(cloud, cluster_id: int):
         return []
 
     token = response.get("back")
-    pages = []
+    lines = []
 
     while token:
-        url = f"{cloud.server}/api/v2/logs/cluster/{cluster_id}/page/{token}"
-        response = curl.sync_request(cloud=cloud, url=url, method="get", data={}, json_output=True)
-        pages.append(response["lines"])
+        url = f"{cloud.server}/api/v2/logs/cluster/{cluster_id}/page/"
+        response = curl.sync_request(
+            cloud=cloud,
+            url=url,
+            method="post",
+            data={"encoded_session": token},
+            json=True,
+            json_output=True,
+        )
+        lines.extend(response["lines"])
         token = response.get("back")
 
-    return pages
+    return sorted(lines, key=lambda line: line["timestamp"])
 
 
-def print_azure_log_pages(pages):
-    for page in pages[::-1]:
-        for line in page:
-            print(
-                format_log_event(
-                    line, instances={}, pretty=True, show_label=True, show_timestamp=True, show_all_timestamps=False
-                )
+def print_azure_log_pages(lines):
+    for line in lines:
+        print(
+            format_log_event(
+                line, instances={}, pretty=True, show_label=True, show_timestamp=True, show_all_timestamps=False
             )
+        )
