@@ -1,7 +1,7 @@
 import re
 import time
 from datetime import datetime, timedelta, timezone
-from typing import List, Optional
+from typing import List, Optional, Tuple
 
 import click
 from rich.console import Console
@@ -9,6 +9,7 @@ from rich.console import Console
 import coiled
 
 from ..utils import CONTEXT_SETTINGS
+from .utils import find_cluster
 
 COLORS = [
     "green",
@@ -24,8 +25,10 @@ COLORS = [
 )
 @click.option(
     "--account",
+    "--workspace",
     default=None,
-    help="Coiled account (uses default account if not specified)",
+    help="Coiled workspace (uses default workspace if not specified)."
+    " Note: --account is deprecated, please use --workspace instead.",
 )
 @click.option(
     "--cluster",
@@ -104,7 +107,7 @@ COLORS = [
     default=3,
     help="Tail polling interval",
 )
-def better_logs(
+def better_logs_cli(
     account: Optional[str],
     cluster: Optional[str],
     no_scheduler: bool,
@@ -120,8 +123,6 @@ def better_logs(
     filter: Optional[str],
     show_all_timestamps: bool,
 ):
-    console = Console(force_terminal=color)
-
     dask = not system or combined
     system = system or combined
     label = label.lower()
@@ -130,47 +131,57 @@ def better_logs(
         raise click.ClickException("You can't use --until when tailing logs.")
 
     with coiled.Cloud(account=account) as cloud:
-        if cluster and cluster.isnumeric():
-            cluster_id = int(cluster)
-        elif cluster:
-            # get cluster by name
-            try:
-                clusters = cloud.get_clusters_by_name(name=cluster)
-                if clusters:
-                    recent_cluster = clusters[-1]
-                else:
-                    raise click.ClickException(f"Unable to find cluster with name '{cluster}'")
+        cluster_info = find_cluster(cloud, cluster or "")
 
-                if tail and recent_cluster["current_state"]["state"] in (
-                    "stopped",
-                    "error",
-                ):
-                    tail = False
-                    console.print(
-                        f"[red]Cluster state is {recent_cluster['current_state']['state']} so not tailing.[/red]"
-                    )
+    instance_labels_dict = make_instance_labels(
+        label=label,
+        no_scheduler=no_scheduler,
+        scheduler_id=cluster_info.get("scheduler", {}).get("instance", {}).get("id"),
+        workers=workers,
+        workers_info=cluster_info["workers"],
+    )
 
-                cluster_id = recent_cluster["id"]
+    console = Console(force_terminal=color)
 
-            except coiled.errors.DoesNotExist:
-                cluster_id = None
-        else:
-            # default to most recent cluster
-            clusters = coiled.list_clusters(max_pages=1)
-            if not clusters:
-                raise ValueError("Unable to find any clusters for your account")
-            match = max(clusters, key=lambda c: c["id"])
-            cluster_id = match["id"]
+    cluster_id = cluster_info["id"]
+    cluster_name = cluster_info["name"]
+    cluster_state = cluster_info["current_state"]["state"]
 
-        if not cluster_id:
-            raise click.ClickException(f"Unable to find cluster `{cluster}`")
+    if tail and cluster_state in (
+        "stopped",
+        "error",
+    ):
+        tail = False
+        console.print(f"[red]Cluster state is {cluster_state} so not tailing.[/red]")
 
-        cluster_info = cloud.cluster_details(cluster_id)
+    if workers and not no_scheduler and not instance_labels_dict:
+        # exit here if there are no instances, otherwise passing empty list to
+        # coiled.better_cluster_logs will result in pulling logs for all instances
+        console.print("no instances match specified filters")
+        return
 
+    console.print(f"=== Logs for {cluster_name} ({cluster_id}) ===\n")
+    better_logs(
+        cluster_id=cluster_id,
+        instance_labels_dict=instance_labels_dict,
+        show_label=label != "none",
+        show_all_instances=workers == "all" and not no_scheduler,
+        console=console,
+        dask=dask,
+        system=system,
+        tail=tail,
+        interval=interval,
+        since=since,
+        until=until,
+        filter=filter,
+        show_all_timestamps=show_all_timestamps,
+    )
+
+
+def make_instance_labels(label, no_scheduler, scheduler_id, workers, workers_info):
     # instance ID's for which to show logs, key maps to label to use
     instances = {}
-    if not no_scheduler and cluster_info.get("scheduler", {}).get("instance"):
-        scheduler_id = cluster_info["scheduler"]["instance"]["id"]
+    if not no_scheduler and scheduler_id:
         instances[scheduler_id] = {
             "label": "scheduler" if label != "none" else "",
             "color": COLORS[-1],
@@ -210,23 +221,37 @@ def better_logs(
 
             return False
 
-        instances.update(
-            {
-                worker["instance"]["id"]: dict(label=worker_label(worker), color=COLORS[idx % len(COLORS)])
-                for idx, worker in enumerate(cluster_info["workers"])
-                if worker.get("instance") and (workers == "all" or filter_worker(worker))
-            }
-        )
+        instances.update({
+            worker["instance"]["id"]: dict(label=worker_label(worker), color=COLORS[idx % len(COLORS)])
+            for idx, worker in enumerate(workers_info)
+            if worker.get("instance") and (workers == "all" or filter_worker(worker))
+        })
 
-    show_all_instances = workers == "all" and not no_scheduler
+    return instances
 
-    console.print(f"=== Logs for {cluster_info['name']} ({cluster_id}) ===\n")
 
-    if not instances:
-        # exit here if there are no instances, otherwise passing empty list to
-        # coiled.better_cluster_logs will result in pulling logs for all instances
-        console.print("no instances match specified filters")
-        return
+def better_logs(
+    *,
+    cluster_id: int,
+    instance_labels_dict: dict,
+    show_label: bool,
+    show_all_instances: bool,
+    console=None,
+    color: bool = True,
+    dask: bool = True,
+    system: bool = False,
+    tail: bool = False,
+    tail_max_times: Optional[int] = None,
+    interval: int = 3,
+    since: Optional[str] = None,
+    until: Optional[str] = None,
+    filter: Optional[str] = None,
+    show_timestamp: bool = True,
+    show_all_timestamps: bool = False,
+    start_sentinel: str = "",
+    stop_sentinel: str = "",
+):
+    console = console or Console(force_terminal=color)
 
     from_timestamp = ts_ms_from_string(since)
     until_timestamp = ts_ms_from_string(until)
@@ -237,12 +262,14 @@ def better_logs(
         current_ms = int(time.time_ns() // 1e6)
         from_timestamp = current_ms - (30 * 1000)
 
+    waiting_for_start_sentinel = bool(start_sentinel)
+
     while True:
         events = coiled.better_cluster_logs(
             cluster_id=cluster_id,
             # function returns all instances if none specified, we'll use that
             # in order to not exclude instances that show up while tailing
-            instance_ids=None if show_all_instances else list(instances.keys()),
+            instance_ids=None if show_all_instances else list(instance_labels_dict.keys()),
             dask=dask,
             system=system,
             since_ms=from_timestamp,
@@ -254,56 +281,122 @@ def better_logs(
             events = [e for e in events if e["timestamp"] != from_timestamp or event_dedupe_key(e) not in last_events]
 
         if events:
-            print_events(console, events, instances, show_all_timestamps=show_all_timestamps)
-
             from_timestamp = events[-1]["timestamp"]
             last_events = {event_dedupe_key(e) for e in events if e["timestamp"] == from_timestamp}
 
-        if tail:
+            # filter using sentinels
+            events, waiting_for_start_sentinel, found_stop_sentinel = filter_events(
+                events, start_sentinel, stop_sentinel, waiting_for_start_sentinel
+            )
+
+            print_events(
+                console=console,
+                events=events,
+                instances=instance_labels_dict,
+                show_label=show_label,
+                show_timestamp=show_timestamp,
+                show_all_timestamps=show_all_timestamps,
+            )
+
+            if found_stop_sentinel:
+                break
+
+        if tail and (tail_max_times is None or tail_max_times > 0):
             # TODO stop tailing once cluster is stopped/errored (future MR)
+            if tail_max_times:
+                tail_max_times -= 1
             time.sleep(interval)
+        elif not tail and until_timestamp is None and events:
+            # if there's no specified end time of range, then it's possible there are more events we want after
+            # the last event that we got back, so make another request with the updated start time of range
+            continue
         else:
             break
 
 
-def match_cluster(clusters: List[dict], cluster: str) -> dict:
-    if cluster.isnumeric():
-        matches = [c for c in clusters if c["id"] == int(cluster)]
-        if len(matches) == 1:
-            return matches[0]
-        elif len(matches) > 1:
-            raise ValueError(f"Multiple clusters match '{cluster}'")
+def filter_events(events, start_sentinel, stop_sentinel, waiting_for_start_sentinel) -> Tuple[list, bool, bool]:
+    found_stop_sentinel = False
 
-    # try to match on cluster name
-    matches = sorted([c for c in clusters if c["name"] == cluster], key=lambda c: c["id"])
-    if matches:
-        return matches[-1]
+    if start_sentinel or stop_sentinel:
+        show_from = 0
+        show_to = -1
+        for i, event in enumerate(events):
+            if waiting_for_start_sentinel and start_sentinel in event["message"]:
+                waiting_for_start_sentinel = False
+                # remove everything before (first) start sentinel
+                # handle sentinel inside longer message
+                pos = event["message"].find(start_sentinel)
+                event["message"] = event["message"][pos + len(start_sentinel) :].strip()
+                # if there's text after start sentinel, include event with sentinel, otherwise start at next
+                # this is especially important because multiple lines can be joined into one event
+                show_from = i if event["message"] else i + 1
 
-    raise ValueError(f"No clusters match '{cluster}'")
+            if show_to == -1 and stop_sentinel in event["message"]:
+                found_stop_sentinel = True
+                # remove everything after (first) stop sentinel
+                # handle sentinel inside longer message
+                pos = event["message"].find(stop_sentinel)
+                event["message"] = event["message"][:pos].strip()
+                # if there's text before stop sentinel, include event with sentinel, otherwise stop at previous
+                show_to = i + 1 if event["message"] else i
+
+            if not waiting_for_start_sentinel and show_to > -1:
+                # minor optimization
+                break
+        if waiting_for_start_sentinel:
+            events = []
+        elif show_from or show_to > -1:
+            events = events[show_from:show_to]
+
+    return events, waiting_for_start_sentinel, found_stop_sentinel
 
 
 def event_dedupe_key(event):
     return f'{event["timestamp"]}#{event["instance_id"]}#{event["message"]}'
 
 
-def print_events(console, events, instances: dict, pretty=True, show_all_timestamps=False):
+def print_events(
+    console,
+    events: List[dict],
+    instances: dict,
+    pretty=True,
+    show_label=True,
+    show_timestamp=True,
+    show_all_timestamps=False,
+):
     for e in events:
-        console.print(format_log_event(e, instances, pretty=pretty, show_all_timestamps=show_all_timestamps))
+        console.print(
+            format_log_event(
+                e,
+                instances,
+                pretty=pretty,
+                show_label=show_label,
+                show_timestamp=show_timestamp,
+                show_all_timestamps=show_all_timestamps,
+            )
+        )
 
 
-def format_log_event(event: dict, instances: dict, pretty: bool, show_all_timestamps: bool) -> str:
+def format_log_event(
+    event: dict, instances: dict, pretty: bool, show_label: bool, show_timestamp: bool, show_all_timestamps: bool
+) -> str:
     message = event["message"]
-    if event["instance_id"] in instances:
-        label = instances[event["instance_id"]]["label"]
-        color = instances[event["instance_id"]]["color"]
+
+    if show_label:
+        if event.get("instance_id") and event.get("instance_id") in instances:
+            label = instances[event["instance_id"]]["label"]
+            color = instances[event["instance_id"]]["color"]
+        else:
+            # we might not know about instance if it showed up while we're tailing all worker logs
+            label = event.get("instance_id") or event.get("instance")
+            color = COLORS[0]
     else:
-        # we might not know about instance if it showed up while we're tailing all worker logs
-        label = event["instance_id"]
-        color = COLORS[0]
+        label = ""
+        color = ""
 
     time_string = ""
 
-    if event.get("timestamp"):
+    if show_timestamp and event.get("timestamp"):
         time_format = "%Y-%m-%d %H:%M:%S.%f"
         t = datetime.utcfromtimestamp(event["timestamp"] / 1000)
 
