@@ -89,10 +89,10 @@ def env():
     " Example: -i coiled -i pytorch",
 )
 @click.option(
-    "--enable-experimental-installer/--disable-experimental-installer",
-    default=None,
+    "--disable-uv-installer",
+    default=False,
     is_flag=True,
-    help="Enable experimental usage of uv to install for a pypi packages.",
+    help="Do not use uv to install PyPI packages when building this environment.",
 )
 def create(
     name,
@@ -106,7 +106,7 @@ def create(
     architecture,
     region_name,
     include_local_code,
-    enable_experimental_installer,
+    disable_uv_installer,
     ignore_local_package,
 ):
     """Create a Coiled software environment"""
@@ -123,7 +123,7 @@ def create(
             region_name=region_name,
             include_local_code=include_local_code,
             ignore_local_packages=ignore_local_package,
-            enable_experimental_installer=enable_experimental_installer,
+            use_uv_installer=not disable_uv_installer,
             use_entrypoint=not ignore_container_entrypoint,
         )
     except BuildError as e:
