@@ -19,7 +19,7 @@ from rich.console import Console
 from urllib3.util import parse_url
 
 import coiled
-from coiled.credentials.google import get_long_lived_adc_to_forward
+from coiled.credentials.google import send_application_default_credentials
 from coiled.spans import span
 from coiled.utils import (
     dict_from_key_val_list,
@@ -466,6 +466,15 @@ def get_entrypoint(connection, container_name) -> str:
     is_flag=True,
     hidden=True,
 )
+@click.option(
+    "--no-credential-forwarding",
+    default=False,
+    is_flag=True,
+    help=(
+        "Disable automatic forwarding of local cloud credentials; "
+        "use if you want your code to use credentials attached to the VM (e.g., AWS Instance Profile)."
+    ),
+)
 @click.option(
     "--forward-gcp-adc",
     default=False,
@@ -485,6 +494,12 @@ def get_entrypoint(connection, container_name) -> str:
     multiple=True,
     help="Paths to not sync when using ``--sync``.",
 )
+@click.option(
+    "--mount-bucket",
+    default=None,
+    multiple=True,
+    help="S3 or GCS bucket(s) to mount as volumes.",
+)
 @click.argument("command", nargs=-1)
 def run(
     name: Optional[str],
@@ -506,15 +521,17 @@ def run(
     sync: bool,
     root: bool,
     skip_entrypoint: bool,
+    no_credential_forwarding: bool,
     forward_gcp_adc: bool,
     tag,
     sync_ignore,
+    mount_bucket,
     command,
 ):
     """
     Run a command on the cloud.
     """
-    start_run(
+    info = start_run(
         name=name,
         workspace=account,
         software=software,
@@ -536,9 +553,12 @@ def run(
         allow_ssh_from=allow_ssh_from,
         sync=sync,
         root=root,
+        no_credential_forwarding=no_credential_forwarding,
         forward_gcp_adc=forward_gcp_adc,
         sync_ignore=sync_ignore,
+        mount_bucket=mount_bucket,
     )
+    return info["exit_code"]
 
 
 def start_run(
@@ -566,8 +586,10 @@ def start_run(
     allow_ssh_from: str = "me",
     sync: bool = False,
     root: bool = False,
+    no_credential_forwarding: bool = False,
     forward_gcp_adc: bool = False,
     sync_ignore: Optional[List[str]] = None,
+    mount_bucket: Optional[List[str]] = None,
 ):
     runtime_env_dict = dict_from_key_val_list(env)
     tags = dict_from_key_val_list(tag)
@@ -593,7 +615,7 @@ def start_run(
     keepalive = parse_timedelta(keepalive)
     shutdown_on_close = keepalive is None
 
-    info = {"command": command, "keepalive": keepalive, "subdomain": subdomain}
+    info = {"command": command, "keepalive": keepalive, "subdomain": subdomain, "exit_code": 0}
 
     success = True
     exception = None
@@ -648,12 +670,17 @@ def start_run(
                     "scheduler_disk_size": disk_size,
                     "worker_disk_size": disk_size,
                     "dashboard_custom_subdomain": subdomain,
+                    "mount_bucket": mount_bucket,
                 }
                 cluster_kwargs["name"] = name or f"run-{short_random_string()}"
                 cluster_kwargs["cloud"] = cloud
                 cluster_kwargs["custom_widget"] = widget  # `Cluster` will update widget until it's "ready"
 
-                dask.config.set({"coiled.use_aws_creds_endpoint": True})
+                if no_credential_forwarding:
+                    cluster_kwargs["credentials"] = None
+                    dask.config.set({"coiled.use_aws_creds_endpoint": False})
+                else:
+                    dask.config.set({"coiled.use_aws_creds_endpoint": True})
                 with coiled.Cluster(**cluster_kwargs) as cluster:
                     info["cluster_id"] = cluster.cluster_id
 
@@ -661,6 +688,9 @@ def start_run(
                     runtime_env_dict["DASK_COILED__SOFTWARE"] = cluster._software_environment_name
                     runtime_env_dict["DASK_COILED__SOFTWARE_REQUIRES_VM_ARCH"] = cluster.arch.vm_arch
 
+                    # by default "shutdown on close" is also what enables "keepalive" timeout on clusters,
+                    # but we want to use "keepalive" timeout (together with "keepalive sessions") to shut
+                    # down cluster after the run (submitted via SSH) is done
                     if not shutdown_on_close:
                         cluster.set_keepalive(keepalive=keepalive)
 
@@ -682,12 +712,10 @@ def start_run(
                     if sync:
                         start_sync(cloud, cluster.cluster_id, console=widget.live.console, ignores=sync_ignore)
 
-                    in_container_files = {}
-
                     if forward_gcp_adc:
-                        gcp_creds_to_forward = get_long_lived_adc_to_forward()
-                        runtime_env_dict = {**gcp_creds_to_forward.get("env", {}), **runtime_env_dict}
-                        in_container_files = {**gcp_creds_to_forward.get("files", {}), **in_container_files}
+                        gcp_creds_to_forward = send_application_default_credentials(cluster, return_creds=True)
+                        if gcp_creds_to_forward:
+                            runtime_env_dict = {**gcp_creds_to_forward.get("env", {}), **runtime_env_dict}
 
                     connection, container_command, results = run_via_ssh(
                         cloud=cloud,
@@ -702,7 +730,6 @@ def start_run(
                         env_unset=list(unset_single_thread_defaults().keys()),
                         workdir=SYNC_TARGET if sync else "/scratch",
                         as_root=root,
-                        in_container_files=in_container_files,
                         console=widget.live.console,
                     )
 
@@ -732,6 +759,7 @@ def start_run(
         return info
     except Exception as e:
         success = False
+        info["exit_code"] = 1
         exception = e
         print("[red]There was an unexpected error:[/red]")
         print(exception)
