@@ -22,7 +22,7 @@ except ImportError:
 logger = logging.getLogger(__name__)
 
 
-def get_gcp_local_session_token(scopes=None) -> dict:
+def get_gcp_local_session_token(scopes=None, set_local_token_env: bool = False) -> dict:
     auth_logger = None
     comp_logger = None
     auth_logger_level = 0
@@ -30,12 +30,29 @@ def get_gcp_local_session_token(scopes=None) -> dict:
 
     try:
         import google.auth
+        from google.auth.exceptions import RefreshError
         from google.auth.transport.requests import Request
 
         with supress_logs(["google.auth._default", "google.auth.compute_engine._metadata"]):
             # get token from local auth
             local_creds, project = google.auth.default(scopes=scopes)
-            local_creds.refresh(Request())
+            try:
+                local_creds.refresh(Request())
+            except RefreshError:
+                # when local creds are service account, we need to explicitly specify scope(s)
+                # or else we get this exception:
+                #   RefreshError:
+                #   ('invalid_scope: Invalid OAuth scope or ID token audience provided.',
+                #    {'error': 'invalid_scope',
+                #     'error_description': 'Invalid OAuth scope or ID token audience provided.'})
+                local_creds, project = google.auth.default(
+                    scopes=[*(scopes or []), "https://www.googleapis.com/auth/cloud-platform"]
+                )
+                local_creds.refresh(Request())
+
+        if set_local_token_env:
+            # set local env var with token, in case `CoiledShippedCredentials` is being used locally
+            os.environ["COILED_LOCAL_CLOUDSDK_AUTH_ACCESS_TOKEN"] = local_creds.token
 
         return {
             "token": local_creds.token,
@@ -97,14 +114,14 @@ def get_long_lived_adc_to_forward(scopes=None):
     }
 
     to_forward["files"] = {
-        "~/.config/gcloud/application_default_credentials.json": adf_json,
-        "~/.config/gcloud/configurations/config_default": default_config,
+        "/gcloud-config/application_default_credentials.json": adf_json,
+        "/gcloud-config/configurations/config_default": default_config,
     }
 
     return to_forward
 
 
-def send_application_default_credentials(cluster, scopes=None):
+def send_application_default_credentials(cluster, scopes=None, return_creds: bool = False):
     to_forward = get_long_lived_adc_to_forward(scopes)
 
     def write_adf_files():
@@ -114,17 +131,29 @@ def send_application_default_credentials(cluster, scopes=None):
             with open(abs_path, "w") as f:
                 f.write(content)
 
+        os.makedirs(os.path.expanduser("~/.config"), exist_ok=True)
+        container_adc_dir = os.path.expanduser("~/.config/gcloud")
+
+        # files are written to /gcloud-config, which is then symlinked to ~/.config/gcloud
+        if not os.path.exists(container_adc_dir):
+            os.symlink("/gcloud-config", container_adc_dir, target_is_directory=True)
+        # otherwise, maybe we should copy the files to both places?
+
     cluster.send_private_envs(to_forward["env"])
-    with Client(cluster) as client:
+    with Client(cluster, name="non-user-write-gcp-adf") as client:
         client.run_on_scheduler(write_adf_files)
         client.run(write_adf_files)
 
+    cluster._queue_cluster_event("credentials", "Google Application Default Credentials forwarded")
     print(
         "Google Application Default Credentials have been written to a file on your Coiled VM(s).\n"
         "These credentials will potentially be valid until explicitly revoked by running\n"
         "gcloud auth application-default revoke"
     )
 
+    if return_creds:
+        return to_forward
+
 
 class CoiledShippedCredentials(GoogleOAuth2Credentials):  # type: ignore
     def __init__(
