@@ -103,6 +103,7 @@ async def scan_conda(prefix: Path, progress: Optional[Progress] = None) -> typin
                 if r:
                     packages.append(r)
         else:
+            logger.info(f"Scanning {len(conda_packages)} conda packages...")
             packages = [
                 pkg for pkg in await asyncio.gather(*[handle_conda_package(pkg) for pkg in conda_packages]) if pkg
             ]
@@ -130,7 +131,7 @@ async def handle_conda_package(pkg: CondaPackage) -> Optional[PackageInfo]:
             return None
         else:
             dist = ResilientDistribution(pkg.prefix / metadata_location)  # type: ignore
-            name = dist.metadata["Name"] or pkg.name
+            name = dist.metadata.get("Name") or pkg.name
             path = Path(str(dist._path))
     else:
         name = pkg.name
@@ -153,7 +154,12 @@ async def handle_conda_package(pkg: CondaPackage) -> Optional[PackageInfo]:
 
 async def handle_dist(dist: Distribution, locations: List[Path]) -> Optional[Union[PackageInfo, CondaPlaceHolder]]:
     # Sometimes the dist name is blank (seemingly only on Windows?)
-    if not dist.name:
+    try:
+        if not dist.name:
+            return
+    # Newer versions of importlib_metadata will raise a KeyError instead
+    # of returning None
+    except KeyError:
         return
     installer = dist.read_text("INSTALLER") or ""
     installer = installer.rstrip()
@@ -328,6 +334,7 @@ async def scan_pip(
         ):
             packages.append(await task)
     else:
+        logger.info(f"Scanning {len(dists)} python packages...")
         packages = await asyncio.gather(*(handle_dist(dist, locations) for dist in dists))
 
     # Resolve duplicate packages
