@@ -4,17 +4,12 @@ import platform
 import re
 import shutil
 import subprocess
-import sys
-import warnings
-import weakref
 from collections import defaultdict
 from concurrent.futures import ThreadPoolExecutor
 from contextlib import asynccontextmanager
 from logging import getLogger
 from pathlib import Path
 from sys import executable
-from tempfile import TemporaryDirectory as TemporaryDirectoryBase
-from tempfile import mkdtemp
 from threading import Lock
 from typing import Dict, List, Optional, TextIO, Tuple, Union, cast
 from urllib.parse import urlparse
@@ -29,7 +24,9 @@ from urllib3.util import parse_url
 from coiled.context import track_context
 from coiled.types import PackageInfo, PackageLevelEnum, ResolvedPackageInfo
 from coiled.utils import (
+    TemporaryDirectory,
     get_encoding,
+    get_temp_dir,
     partition,
     recurse_importable_python_files,
     validate_wheel,
@@ -54,64 +51,6 @@ DEFAULT_JSON_PYPI_URL = "https://pypi.org/pypi"
 PYTHON_VERSION = platform.python_version_tuple()
 LOCK_PATH = Path(os.path.expanduser("~")) / ".coiled" / "locks"
 
-# ignore_cleanup_errors was added in 3.10
-if sys.version_info < (3, 10):
-
-    class TemporaryDirectory(TemporaryDirectoryBase):
-        def __init__(self, suffix=None, prefix=None, dir=None, ignore_cleanup_errors=False):
-            self.name = mkdtemp(suffix, prefix, dir)
-            self._ignore_cleanup_errors = ignore_cleanup_errors
-            self._finalizer = weakref.finalize(
-                self,
-                self._cleanup,
-                self.name,
-                warn_message="Implicitly cleaning up {!r}".format(self),
-                ignore_errors=self._ignore_cleanup_errors,
-            )
-
-        @classmethod
-        def _rmtree(cls, name, ignore_errors=False):
-            def onerror(func, path, exc_info):
-                if issubclass(exc_info[0], PermissionError):
-
-                    def resetperms(path):
-                        try:
-                            os.chflags(path, 0)
-                        except AttributeError:
-                            pass
-                        os.chmod(path, 0o700)
-
-                    try:
-                        if path != name:
-                            resetperms(os.path.dirname(path))
-                        resetperms(path)
-                        try:
-                            os.unlink(path)
-                        # PermissionError is raised on FreeBSD for directories
-                        except (IsADirectoryError, PermissionError):
-                            cls._rmtree(path, ignore_errors=ignore_errors)
-                    except FileNotFoundError:
-                        pass
-                elif issubclass(exc_info[0], FileNotFoundError):
-                    pass
-                else:
-                    if not ignore_errors:
-                        raise
-
-            shutil.rmtree(name, onerror=onerror)
-
-        @classmethod
-        def _cleanup(cls, name, warn_message, ignore_errors=False):
-            cls._rmtree(name, ignore_errors=ignore_errors)
-            warnings.warn(warn_message, ResourceWarning, stacklevel=2)
-
-        def cleanup(self):
-            if self._finalizer.detach() or os.path.exists(self.name):
-                self._rmtree(self.name, ignore_errors=self._ignore_cleanup_errors)
-
-else:
-    TemporaryDirectory = TemporaryDirectoryBase
-
 
 async def create_subprocess_exec(
     program: str,
@@ -142,8 +81,9 @@ def partition_ignored_packages(
 def partition_local_python_code_packages(packages: List[PackageInfo]) -> Tuple[List[PackageInfo], List[PackageInfo]]:
     return partition(
         packages,
-        lambda pkg: pkg["name"].startswith(COILED_LOCAL_PACKAGE_PREFIX)
-        and not cast(str, pkg["wheel_target"]).endswith(".whl"),
+        lambda pkg: (
+            pkg["name"].startswith(COILED_LOCAL_PACKAGE_PREFIX) and not cast(str, pkg["wheel_target"]).endswith(".whl")
+        ),
     )
 
 
@@ -196,7 +136,7 @@ async def create_wheel(pkg_name: str, version: str, src: str) -> ResolvedPackage
         (
             FileLock(LOCK_PATH / ("." + pkg_name + version + ".build-lock")),
             Lock(),
-            TemporaryDirectory(ignore_cleanup_errors=True),
+            get_temp_dir(ignore_cleanup_errors=True),
         ),
     )
     async with async_lock(package_lock, thread_lock):
@@ -266,7 +206,7 @@ async def create_wheel(pkg_name: str, version: str, src: str) -> ResolvedPackage
 
 @track_context
 async def create_wheel_from_egg(pkg_name: str, version: str, src: str) -> ResolvedPackageInfo:
-    tmpdir = TemporaryDirectory(ignore_cleanup_errors=True)
+    tmpdir = get_temp_dir(ignore_cleanup_errors=True)
     outdir = Path(tmpdir.name) / Path(pkg_name)
     outdir.mkdir(parents=True)
     logger.debug(f"Attempting to create a wheel for {pkg_name} in directory {src}")
@@ -333,7 +273,7 @@ async def create_wheel_from_src_dir(pkg_name: str, version: str, src: str) -> Re
         (
             FileLock(LOCK_PATH / (f".{pkg_name}{version}.build-lock")),
             Lock(),
-            TemporaryDirectory(ignore_cleanup_errors=True),
+            get_temp_dir(ignore_cleanup_errors=True),
         ),
     )
     async with async_lock(package_lock, thread_lock):
@@ -583,7 +523,7 @@ def _get_pip_index_urls() -> List[str]:
 
     extra_index_urls = sorted(set(extra_index_urls) - {index_url})
 
-    return [index_url, *extra_index_urls]
+    return [url.strip() for url in [index_url, *extra_index_urls]]
 
 
 def get_index_urls():
@@ -612,7 +552,7 @@ def get_index_urls():
                     get_netrc_auth(index_url)
                     # keyring could have URL stored by full URL or netloc
                     or get_keyring_auth(parsed_url._replace(auth=None).url, username)
-                    or get_keyring_auth(parsed_url.netloc, username)
+                    or get_keyring_auth(parsed_url.netloc or "", username)
                 )
                 if auth_parts is not None:
                     username, password = auth_parts
