@@ -11,21 +11,24 @@ import os
 import platform
 import random
 import re
+import shutil
 import ssl
 import string
 import sys
-import tempfile
 import threading
 import time
 import traceback
 import uuid
 import warnings
+import weakref
 from copy import deepcopy
 from datetime import datetime, timedelta, timezone
 from hashlib import md5
 from logging.config import dictConfig
 from math import ceil
 from pathlib import Path
+from tempfile import TemporaryDirectory as TemporaryDirectoryBase
+from tempfile import mkdtemp
 from typing import Callable, Dict, Iterable, List, NoReturn, Optional, Set, Tuple, Union
 from urllib.parse import unquote, urlencode, urlparse
 from zipfile import PyZipFile
@@ -80,11 +83,11 @@ COILED_SERVER = "https://cloud.coiled.io"
 COILED_RUNTIME_REGEX = re.compile(r"^coiled/coiled-runtime-(?P<version>\d+\-\d+\-\d+)-*")
 
 # Dots after family names ensure we do not match t3a or other variants
-AWS_BALANCED_INSTANCES_FAMILY_FILTER = ("t2.", "t3.", "m5.", "m6i.", "m7i.", "m7i-flex.", "t4g.", "m6g.", "m7g.")
-AWS_RECOMMEND_BALANCED_INSTANCES_FAMILY_FILTER = ("m5.", "m6i.", "m6g.", "m7g.")
-AWS_INSTANCES_FAMILY_FILTER = AWS_BALANCED_INSTANCES_FAMILY_FILTER  # deprecated
-AWS_GPU_INSTANCE_FAMILIES_FILTER = ("g4dn",)
-AWS_UNBALANCED_INSTANCE_FAMILIES_FILTER = ("c5.", "c6i.", "r6i.", "c7g.", "r7g.")
+AWS_BALANCED_NOT_RECOMMENDED = r"t\dg?\.|m\di-flex\."
+AWS_RECOMMEND_BALANCED_INSTANCES_FILTER = r"m[56789][ig]?\."  # match m5, m6i, m7g, etc
+AWS_BALANCED_INSTANCES_FAMILY_FILTER = f"{AWS_RECOMMEND_BALANCED_INSTANCES_FILTER}|{AWS_BALANCED_NOT_RECOMMENDED}"
+AWS_GPU_INSTANCE_FAMILIES_FILTER = "g4dn"
+AWS_UNBALANCED_INSTANCE_FAMILIES_FILTER = r"(c[56789]|r[6789])[ig]?\."  # match c5, c6i, c7g, r7i, r7g, etc
 
 GCP_SCHEDULER_GPU = {
     "scheduler_accelerator_type": "nvidia-tesla-t4",
@@ -158,6 +161,77 @@ def random_str(length: int = 8):
     return "".join(random.choices(string.ascii_lowercase + string.digits, k=length))
 
 
+# ignore_cleanup_errors was added in 3.10
+if sys.version_info < (3, 10):
+
+    class TemporaryDirectory(TemporaryDirectoryBase):
+        def __init__(self, suffix=None, prefix=None, dir=None, ignore_cleanup_errors=False):
+            self.name = mkdtemp(suffix, prefix, dir)
+            self._ignore_cleanup_errors = ignore_cleanup_errors
+            self._finalizer = weakref.finalize(
+                self,
+                self._cleanup,
+                self.name,
+                warn_message="Implicitly cleaning up {!r}".format(self),
+                ignore_errors=self._ignore_cleanup_errors,
+            )
+
+        @classmethod
+        def _rmtree(cls, name, ignore_errors=False):
+            def onerror(func, path, exc_info):
+                if issubclass(exc_info[0], PermissionError):
+
+                    def resetperms(path):
+                        try:
+                            os.chflags(path, 0)
+                        except AttributeError:
+                            pass
+                        os.chmod(path, 0o700)
+
+                    try:
+                        if path != name:
+                            resetperms(os.path.dirname(path))
+                        resetperms(path)
+                        try:
+                            os.unlink(path)
+                        # PermissionError is raised on FreeBSD for directories
+                        except (IsADirectoryError, PermissionError):
+                            cls._rmtree(path, ignore_errors=ignore_errors)
+                    except FileNotFoundError:
+                        pass
+                elif issubclass(exc_info[0], FileNotFoundError):
+                    pass
+                else:
+                    if not ignore_errors:
+                        raise
+
+            shutil.rmtree(name, onerror=onerror)
+
+        @classmethod
+        def _cleanup(cls, name, warn_message, ignore_errors=False):
+            cls._rmtree(name, ignore_errors=ignore_errors)
+            warnings.warn(warn_message, ResourceWarning, stacklevel=2)
+
+        def cleanup(self):
+            if self._finalizer.detach() or os.path.exists(self.name):
+                self._rmtree(self.name, ignore_errors=self._ignore_cleanup_errors)
+
+else:
+    TemporaryDirectory = TemporaryDirectoryBase
+
+
+def get_temp_dir(suffix=None, prefix=None, dir=None, ignore_cleanup_errors=False) -> TemporaryDirectory:
+    """Return a TemporaryDirectory that is more likely to not have write issues"""
+    try:
+        tempdir = TemporaryDirectory(suffix=suffix, prefix=prefix, dir=dir, ignore_cleanup_errors=ignore_cleanup_errors)
+    except IOError:
+        # If the usual temporary directory paths are not writable, try the current directory
+        tempdir = TemporaryDirectory(
+            suffix=suffix, prefix=prefix, dir=os.getcwd(), ignore_cleanup_errors=ignore_cleanup_errors
+        )
+    return tempdir
+
+
 class GatewaySecurity(Security):
     """A security implementation that temporarily stores credentials on disk.
 
@@ -177,7 +251,7 @@ class GatewaySecurity(Security):
     def get_connection_args(self, role):
         ctx = None
         if self.tls_scheduler_key and self.tls_scheduler_cert:
-            with tempfile.TemporaryDirectory() as tempdir:
+            with get_temp_dir() as tempdir:
                 key_path = os.path.join(tempdir, "dask.pem")
                 cert_path = os.path.join(tempdir, "dask.crt")
                 with open(key_path, "w") as f:
@@ -200,6 +274,8 @@ async def handle_api_exception(response, exception_cls: Type[Exception] = Server
         # First see if it's an error we created that has a more useful
         # message
         error_body = await response.json()
+        if error_body.get("code") == PermissionsError.code:
+            exception_cls = PermissionsError
         if isinstance(error_body, dict):
             errs = error_body.get("non_field_errors")
             if "aren't a member" in error_body.get("detail", ""):
@@ -393,17 +469,12 @@ async def handle_credentials(
             "please reach out to Coiled Support at support@coiled.io"
         )
 
-    # TODO: revert when we remove versioneer
-    if dask.config.get("coiled.no-minimum-version-check", False):
-        client_version = "coiled-frontend-js"
-    else:
-        client_version = COILED_VERSION
     account_usage = {}
     # Validate token and get username
     async with aiohttp.ClientSession(
         headers={
             "Authorization": get_auth_header_value(token),
-            "Client-Version": client_version,
+            "Client-Version": COILED_VERSION,
         }
     ) as session:
         try:
@@ -1126,19 +1197,19 @@ def _get_preferred_instance_types_from_cpu_memory(
         ]
     elif backend == "aws":
         aws_family_filter = (
-            AWS_RECOMMEND_BALANCED_INSTANCES_FAMILY_FILTER if recommended else AWS_BALANCED_INSTANCES_FAMILY_FILTER
+            AWS_RECOMMEND_BALANCED_INSTANCES_FILTER if recommended else AWS_BALANCED_INSTANCES_FAMILY_FILTER
         )
         if gpus:
             aws_family_filter = AWS_GPU_INSTANCE_FAMILIES_FILTER
         elif include_unbalanced:
-            aws_family_filter += AWS_UNBALANCED_INSTANCE_FAMILIES_FILTER
+            aws_family_filter = f"{aws_family_filter}|{AWS_UNBALANCED_INSTANCE_FAMILIES_FILTER}"
 
         instances = [
             (instance_name, instance_specs)
             for instance_name, instance_specs in list_instance_types(
                 cores=cpu, memory=memory, backend=backend, arch=arch
             ).items()
-            if instance_name.startswith(aws_family_filter)
+            if re.match(aws_family_filter, instance_name)
         ]
     else:
         raise UnsupportedBackendError(f"Unexpected backend: {backend}")
@@ -1459,7 +1530,7 @@ def _parse_targets(targets):
 def cluster_firewall(
     target: str,
     *,
-    scheduler: int = 8786,
+    scheduler: int = 443,
     dashboard: Optional[int] = None,
     jupyter: bool = False,
     ssh: bool = False,
@@ -1488,7 +1559,7 @@ def cluster_firewall(
         or use "me" to automatically get your IP address and use that, or
         "everyone" for "0.0.0.0/0".
     scheduler: int
-        Port to open for access to scheduler, 8786 by default.
+        Port to open for access to scheduler, 443 by default.
     dashboard: Optional[int]
         Port to open for direct access to dashboard, closed by default if https is used,
         otherwise 8787.
@@ -1786,6 +1857,8 @@ def short_random_string():
 
 
 def parse_bytes_as_gib(size: Optional[Union[str, int]]) -> Optional[int]:
+    if isinstance(size, str) and size.isnumeric():
+        size = int(size)
     # convert string to GiB, rounding up
     return ceil(dask.utils.parse_bytes(size) / 1073741824) if isinstance(size, str) else size
 
