@@ -34,6 +34,7 @@ from coiled.context import track_context
 from coiled.core import Async, IsAsynchronous, Sync, delete_docstring, list_docstring
 from coiled.core import Cloud as OldCloud
 from coiled.errors import ClusterCreationError, DoesNotExist, ServerError
+from coiled.exceptions import PermissionsError
 from coiled.types import (
     ArchitectureTypesEnum,
     AWSOptions,
@@ -87,6 +88,8 @@ async def handle_api_exception(response, exception_cls=ServerError) -> NoReturn:
         raise exception_cls(
             f"Unexpected status code ({response.status}) to {response.method}:{response.url}, contact support@coiled.io"
         ) from None
+    if error_body.get("code") == PermissionsError.code:
+        exception_cls = PermissionsError
     if "message" in error_body:
         raise exception_cls(error_body["message"])
     if "detail" in error_body:
@@ -521,7 +524,7 @@ class CloudV2(OldCloud, Generic[IsAsynchronous]):
         gpu_enabled: bool = False,
         architecture: ArchitectureTypesEnum = ArchitectureTypesEnum.X86_64,
         region_name: Optional[str] = None,
-        enable_experimental_installer: Optional[bool] = None,
+        use_uv_installer: bool = True,
     ) -> SoftwareEnvironmentAlias:
         workspace = workspace or self.default_workspace
         prepared_packages: List[PackageSchema] = []
@@ -557,7 +560,7 @@ class CloudV2(OldCloud, Generic[IsAsynchronous]):
                 architecture=architecture,
                 gpu_enabled=gpu_enabled,
                 region_name=region_name,
-                enable_experimental_installer=enable_experimental_installer,
+                use_uv_installer=use_uv_installer,
             )
         return result
 
@@ -725,11 +728,12 @@ class CloudV2(OldCloud, Generic[IsAsynchronous]):
                     else:
                         raise ClusterCreationError(error_message)
             else:
+                error_class = PermissionsError if response_json.get("code") == PermissionsError.code else ServerError
                 if "message" in response_json:
-                    raise ServerError(response_json["message"])
+                    raise error_class(response_json["message"])
                 if "detail" in response_json:
-                    raise ServerError(response_json["detail"])
-                raise ServerError(response_json)
+                    raise error_class(response_json["detail"])
+                raise error_class(response_json)
 
         return response_json["id"], response_json["existing"]
 
